package statsapi

import (
	"io"
	"net/http"
	"net/url"
)

var DefaultClient = NewClient(http.DefaultClient)

func RequestSchedule(sportId, teamId string) ([]byte, error) {
	return DefaultClient.RequestSchedule(sportId, teamId)
}

func RequestFeed(gamePk string) ([]byte, error) {
	return DefaultClient.RequestFeed(gamePk)
}

func RequestDiffPatch(gamePk, startTimecode, pushUpdateId string) ([]byte, error) {
	return DefaultClient.RequestDiffPatch(gamePk, startTimecode, pushUpdateId)
}

type Client struct {
	baseURL    url.URL
	httpClient *http.Client
}

func NewClient(c *http.Client) *Client {
	return &Client{
		baseURL: url.URL{
			Scheme: "https",
			Host:   "statsapi.mlb.com",
		},
		httpClient: c,
	}
}

func (c *Client) RequestSchedule(sportId, teamId string) ([]byte, error) {
	endpoint := url.URL{Path: "api/v1/schedule"}
	query := endpoint.Query()
	query.Add("sportId", sportId)
	query.Add("teamId", teamId)
	endpoint.RawQuery = query.Encode()

	url := c.baseURL.ResolveReference(&endpoint)

	return c.get(url.String())
}

func (c *Client) RequestFeed(gamePk string) ([]byte, error) {
	endpoint := url.URL{Path: "api/v1.1/game/" + gamePk + "/feed/live"}
	url := c.baseURL.ResolveReference(&endpoint)

	return c.get(url.String())
}

func (c *Client) RequestDiffPatch(gamePk, startTimecode, pushUpdateId string) ([]byte, error) {
	endpoint := url.URL{Path: "api/v1.1/game/" + gamePk + "/feed/live/diffPatch"}
	query := endpoint.Query()
	query.Add("language", "en")
	query.Add("startTimecode", startTimecode)
	query.Add("pushUpdateId", pushUpdateId)
	endpoint.RawQuery = query.Encode()

	url := c.baseURL.ResolveReference(&endpoint)

	return c.get(url.String())
}

func (c *Client) get(url string) ([]byte, error) {
	req, err := http.NewRequest("GET", url, nil)
	if err != nil {
		return nil, err
	}

	resp, err := c.httpClient.Do(req)
	if err != nil {
		return nil, err
	}
	defer resp.Body.Close()

	body, err := io.ReadAll(resp.Body)
	return body, err
}
